"""
Badge
=====

See: https://www.engie.design/fluid-design-system/components/badge/

A badge should be used to bring a meaningful piece of information out. It may
either be textual or numerical. A badge may represent a status or a number of
unread notifications for example. Contrary to tags, badges cannot be
interactive. And multiple badges are not to be used side by side. A number of
variations are suggested here to help you use accessible badges.
"""
from .base import Node

class Badge(Node):
    """Badge component
    """
    NODE_PROPS = ('variant', 'style', 'uppercase', 'size')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'p'
    "Rendered HTML tag."
    POSSIBLE_VARIANTS = ('subtle', 'minimal')
    "Possible values for variant argument."
    POSSIBLE_STYLES = ('danger', 'warning', 'success', 'information',
            'discovery')
    "Possible values for style argument."
    POSSIBLE_SIZES = ('xs', 'sm', 'lg')
    "Possible values for size argument."

    # Parent Tags can set the arguments of their children Tags, in effect
    # changing their appearance.
    CATCH_PROPS = ('badge_kwargs',)

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('uppercase'), context):
            values['class'].append('nj-badge--uppercase')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-badge--{size}')

        variant = self.eval(self.kwargs.get('variant'), context)
        if variant in self.POSSIBLE_VARIANTS:
            values['class'].append(f'nj-badge--{variant}')

        style = self.eval(self.kwargs.get('style'), context)
        if style in self.POSSIBLE_STYLES:
            values['class'].append(f'nj-badge--{style}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-badge {class}" {props}>
  {label}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Badge': Badge,
}



"""
Link
====

See: https://www.engie.design/fluid-design-system/components/link/

Links are key elements for navigation. They should only be used for this
purpose, and not to trigger specific actions. For the latter case, use a button
instead. Different colors from our design system can be used to highlight
different categories of links.
To improve accessibility, we recommend to always use underscoring so that links
can easily be spotted by users. 
""" # pylint:disable=line-too-long

from .base import Node

class Link(Node):
    """Link component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('icon_before', 'size', 'style', 'external')
    "Extended Template Tag arguments."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."
    POSSIBLE_STYLES = ('bold', 'contextual', 'grayed', 'high-contrast',
            'inverse')
    "Possible values for style argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        style = self.eval(self.kwargs.get('style'), context)
        if style in self.POSSIBLE_STYLES:
            values['class'].append(f'nj-link--{style}')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-link--{size}')

        if 'icon' in self.slots:
            values['class'].append('nj-link-icon')


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.eval(self.kwargs.get('external'), context):
            template = """
<a target="_blank" class="nj-link nj-link-icon {class}" {props}>
  {child}
  <span class="nj-sr-only">&nbsp;(open in new tab)</span>
  <span aria-hidden="true" class="material-icons">open_in_new</span>
</a>
"""
        elif 'icon' in self.slots:
            if self.eval(self.kwargs.get('icon_before'), context):
                template = """
<a class="nj-link nj-link-icon--before {class}" {props}>
  {slot_icon}
  {child}
</a>
"""
            else:
                template = """
<a class="nj-link {class}" {props}>
  {child}
  {slot_icon}
</a>
"""
        else:
            template = """
<a class="nj-link {class}" {props}>
  {child}
</a>
"""
        return self.format(template, values, context)


components = {
    'Link': Link,
}



"""
Modal
=====

See: https://www.engie.design/fluid-design-system/components/modal/

Modal allows you to add dialogs to your site for lightboxes, user notifications,
or completely custom content.
"""
from django.utils.translation import gettext as _
#-
from .base import Node
from .button import Button

class Modal(Node):
    """Modal component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('title', 'footer', 'icon')
    "Named children."
    MODES = ('default', 'information', 'spinner')
    "Available variants."
    NODE_PROPS = ('append_to', 'vcenter', 'size', 'fcenter')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('dialog',)
    "Possible values for style argument."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_close'] = _("Close")
        values['txt_loading'] = _("Loading...")
        values['props'].insert(0, ('role', 'alertDialog'))

        append_to = self.eval(self.kwargs.get('append_to'), context)
        if append_to:
            values['props'].append(('data-appendTo', append_to))

        if self.eval(self.kwargs.get('vcenter'), context):
            values['class'].append('nj-modal--vertical-centered')

        if self.mode == 'information':
            values['class'].append('nj-modal--information')

            context['icon_icon_kwargs'] = {
                'class': 'nj-modal__icon',
                'size': 'xxl',
            }
        elif self.mode == 'spinner':
            values['class'].append('nj-modal--information')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['dialog_class'].append(f'nj-modal--{size}')

        if self.eval(self.kwargs.get('fcenter'), context):
            values['footer_class'].append('nj-modal__footer--centered')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-modal fade {class}" aria-labelledby="{id}-title" {props}>
  <div class="nj-modal__dialog {dialog_class}" role="document" {dialog_props}>
    <div class="nj-modal__content">
      <div class="nj-modal__header">
        {slot_title}

        {% IconButton type="button" class="nj-modal__close" size="lg" data-dismiss="modal" %}
          {txt_close}
          {% Slot 'icon' %}
            <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
              close
            </span>
          {% endSlot %}
        {% endIconButton %}
      </div>
      <div class="nj-modal__body">
        {child}
      </div>
      {slot_footer}
    </div>
  </div>
</div>
"""
        return self.format(template, values, context, is_template=True)


    def render_information(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-modal fade {class}" aria-labelledby="{id}-title" {props}>
  <div class="nj-modal__dialog {dialog_class}" role="document" {dialog_props}>
    <div class="nj-modal__content">
      <div class="nj-modal__header">
        {% IconButton type="button" class="nj-modal__close" size="lg" data-dismiss="modal" %}
          {txt_close}
          {% Slot 'icon' %}
            <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
              close
            </span>
          {% endSlot %}
        {% endIconButton %}
      </div>
      <div class="nj-modal__body">
        {slot_icon}
        {slot_title}
        {child}
      </div>
      {slot_footer}
    </div>
  </div>
</div>
"""
        return self.format(template, values, context, is_template=True)


    def render_spinner(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-modal fade {class}" aria-labelledby="{id}-title" {props}>
  <div class="nj-modal__dialog {dialog_class}" role="document" {dialog_props}>
    <div class="nj-modal__content">
      <div class="nj-modal__body">
        <div class="nj-spinner nj-spinner--md nj-modal__loading-spinner"
            role="status">
          <span class="nj-sr-only">{txt_loading}</span>
        </div>
        {slot_title}
        {child}
      </div>
      {slot_footer}
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_slot_title(self, values, context):
        """Render html of the slot.
        """
        if not values['astag']:
            values['astag'] = 'h1'
        if self.mode == 'default':
            template = """
<{astag} id="{id}-title" class="nj-modal__title {class}" {props}>
  {slot_icon}
  {child}
</{astag}>
"""
        else:
            template = """
<{astag} id="{id}-title" class="nj-modal__title {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


    def render_slot_footer(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<div class="nj-modal__footer {class}" {props}>
  {child}
</div>
"""
        return tmpl.format(**values)


class ModalButton(Button):
    """Modal trigger button component
    """
    NODE_PROPS = ('modal', *Button.NODE_PROPS)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        super().prepare(values, context)

        modal_id = self.eval(self.kwargs['modal'], context)
        values['props'].append(('data-toggle', 'modal'))
        values['props'].append(('data-target', modal_id))


class ModalButtonDismiss(Button):
    """Modal close button component
    """
    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        super().prepare(values, context)

        values['props'].append(('data-dismiss', 'modal'))


components = {
    'Modal': Modal,
    'ModalBtn': ModalButton,
    'ModalCloseBtn': ModalButtonDismiss,
}



"""
Checkbox
========

See: https://www.engie.design/fluid-design-system/components/checkbox/

A checkbox is an input control that allows the user to give a feedback by
choosing several items from a list of options. For example, you can use
checkbox when user may have to select multiple options from a list of items, or
when an explicit action is required to apply the settings in your product.
""" # pylint:disable=line-too-long

import logging
#-
from .base import ChoiceSetNode

_logger = logging.getLogger(__name__)


class CheckboxSet(ChoiceSetNode):
    """Checkbox component
    """
    NODE_PROPS = ('exclude', 'inline')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('item',)
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('inline'), context):
            values['item_class'].append('nj-checkbox--inline')


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<fieldset class="nj-checkbox-group {class}" {props}>
  <legend class="nj-checkbox-group__legend">
    {label}
    {tmpl_errors}
  </legend>
  {tmpl_items}
</fieldset>
"""
        else:
            template = """
<fieldset class="nj-checkbox-group {class}" {props}>
  <legend class="nj-checkbox-group__legend">{label}</legend>
  {tmpl_items}
</fieldset>
"""
        return self.format(template, values, context)


    def render_tmpl_items(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<div class="nj-checkbox {class}">
  <label for="{id}">
    <input type="checkbox" name="{name}" id="{id}" value="{value}" {props}>
    {child}
  </label>
</div>
"""
        excludes = self.eval(self.kwargs.get('exclude', []), context)
        if isinstance(excludes, str):
            excludes = [x.strip() for x in excludes.split(';')]

        items = []
        for ii, (_, val, txt) in enumerate(self.choices()):
            options = {
                'id': '%s-%s' % (values['id'], ii + 1),
                'value': val,
                'child': txt,
                'name': self.bound_field.name,
                'class': values['item_class'],
            }
            props = []
            if self.check_test(val):
                props.append('checked')
            if val in excludes:
                options['class'] += ' nj-checkbox--disabled'
                props.append('disabled')
            if self.bound_field.errors:
                props.append('aria-invalid="true"')
                props.append(
                        'aria-describedby="{id}-errors"'\
                        .format(id=values['id']))
            options['props'] = ' '.join(props)
            items.append(self.format(template, options))

        return '\n'.join(items)


    def render_tmpl_errors(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<span id="{id}-errors" class="nj-checkbox__error">
  {child}
</span>
"""
        child = '\n'.join(self.bound_field.errors)
        return template.format(child=child, id=values['id'])


components = {
    'CheckboxSet': CheckboxSet,
}



"""
Icon Button
===========

See: https://www.engie.design/fluid-design-system/components/icon-button/
"""
from .base import COLORS, Node

class IconButton(Node):
    """IconButton component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('disabled', 'color', 'size')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    POSSIBLE_SIZES = ('xs', 'sm', 'lg')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('disabled'), context):
            values['props'].append(('disabled', ''))

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-icon-btn--{color}')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-icon-btn--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        if values['child']:
            template = """
<{astag} class="nj-icon-btn {class}" {props}>
  <span class="nj-sr-only">{child}</span>
  {slot_icon}
</{astag}>
"""
        else:
            template = """
<{astag} class="nj-icon-btn {class}" {props}>
  {slot_icon}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'IconButton': IconButton,
}



"""
Avatar
======

See: https://www.engie.design/fluid-design-system/components/avatar/

Avatars are used to display a person's picture or initials. Avatars may help in
creating an emotional connection to the product and in validating that the
experience is indeed tailored for the current user.
"""
from django.utils.translation import gettext as _
#-
from .base import Node

class Avatar(Node):
    """Avatar component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'minimal')
    "Available variants."
    NODE_PROPS = ('size', 'initial', 'src', 'alt', 'clickable', 'badge',
            'status')
    "Extended Template Tag arguments."
    POSSIBLE_SIZES = ('sm', 'lg', 'xl')
    "Possible values for size argument."
    POSSIBLE_STATUSES = ('offline', 'away', 'busy', 'online')
    "Possible values for status argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_badge'] = _("notifications")

        src = self.eval(self.kwargs.get('src'), context)
        if src:
            alt = self.eval(self.kwargs.get('alt'), context)
            values['src'] = src
            values['alt'] = alt
        else:
            initial = self.eval(self.kwargs.get('initial'), context)
            if initial:
                values['class'].append('nj-avatar--initials')
            else:
                values['class'].append('nj-avatar--default-icon')
            values['initial'] = initial

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-avatar--{size}')
            values['size'] = size

        if self.eval(self.kwargs.get('clickable'), context):
            values['class'].append('nj-avatar--clickable')
            if 'astag' not in self.kwargs:
                values['astag'] = 'button'

        badge = self.eval(self.kwargs.get('badge'), context)
        try:
            values['badge'] = int(badge)
        except (ValueError, TypeError):
            pass

        status = self.eval(self.kwargs.get('status'), context)
        if status in self.POSSIBLE_STATUSES:
            values['status'] = status


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-avatar {class}" {props}>
  {tmpl_picture}
  {tmpl_initial}
  {tmpl_child}
  {tmpl_badge}
  {tmpl_status}
</{astag}>
"""
        return self.format(template, values, context)


    def render_minimal(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-avatar {class}" {props}>
  <div class="nj-avatar__picture">
    <img src="{src}" alt="{alt}">
  </div>
</{astag}>
"""
        return self.format(template, values, context)


    def render_tmpl_child(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values['child'].strip():
            return ''
        tmpl = '<span class="nj-sr-only">{child}</span>'
        return tmpl.format(**values)


    def render_tmpl_picture(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values.get('src'):
            return ''
        tmpl = """
<img class="nj-avatar__picture" src="{src}" alt="{alt}">
"""
        return tmpl.format(**values)


    def render_tmpl_initial(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values.get('initial'):
            return ''
        tmpl = """
<span class="nj-avatar__initials" aria-hidden="true">{initial}</span>
"""
        return tmpl.format(**values)


    def render_tmpl_badge(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values.get('badge'):
            return ''
        if values.get('size') == 'sm':
            return ''

        badge_class = []

        if values.get('size') == 'xl':
            badge_class.append('nj-badge--lg')

        tmpl = """
<div class="nj-badge nj-badge--information {badge_class}">
  <p>{badge} <span class="nj-sr-only">{txt_badge}</span></p>
</div>
"""
        return tmpl.format(badge_class=' '.join(badge_class), **values)


    def render_tmpl_status(self, values, context):
        """Dynamically render a part of the component's template.
        """
        status = values.get('status')
        if not status:
            return ''

        status_class = []

        if status == 'offline':
            status_class.append('nj-status-indicator--offline')
            txt_status = _("Offline")
        elif status == 'away':
            status_class.append('nj-status-indicator--away')
            txt_status = _("Away")
        elif status == 'busy':
            status_class.append('nj-status-indicator--in-progress')
            txt_status = _("In progress")
        else:
            status_class.append('nj-status-indicator--online')
            txt_status = _("Online")

        size = values.get('size')
        if size == 'xl':
            status_class.append('nj-status-indicator--lg')
        elif size != 'lg':
            status_class.append('nj-status-indicator--sm')

        tmpl = """
<div class="nj-status-indicator {status_class}">
  <div class="nj-status-indicator__svg">
    <span class="nj-sr-only">{txt_status}</span>
  </div>
</div>
"""
        return tmpl.format(status_class=' '.join(status_class),
                txt_status=txt_status, **values)


class AvatarMore(Node):
    """Avatar more items component
    """
    NODE_PROPS = ('count', 'size', 'clickable')
    "Extended Template Tag arguments."
    POSSIBLE_SIZES = ('sm', 'lg', 'xl')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-avatar--{size}')

        count = self.eval(self.kwargs.get('count'), context)
        if count:
            try:
                count = int(count)
            except ValueError:
                count = None

        values['count'] = count
        values['txt_more'] = _("Show {count} more user profiles")\
                .format(count=count)

        clickable = self.eval(self.kwargs.get('clickable'), context)
        if clickable:
            values['class'].append('nj-avatar--clickable')
            if 'astag' not in self.kwargs:
                values['astag'] = 'button'
        values['clickable'] = clickable


    def render_default(self, values, context):
        """Html output of the component
        """
        if values['clickable']:
            template = """
<{astag} class="nj-avatar nj-avatar--remaining-count {class}" {props}>
  <span aria-hidden="true">+{count}</span>
  <span class="nj-sr-only">{txt_more}</span>
</{astag}>
"""
        else:
            template = """
<{astag} class="nj-avatar nj-avatar--remaining-count {class}" {props}>
  +{count}
</{astag}>
"""
        return self.format(template, values)


class AvatarList(Node):
    """Avatar list component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('variant',)
    "Extended Template Tag arguments."
    POSSIBLE_VARIANTS = ('compact',)
    "Possible values for variant argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        variant = self.eval(self.kwargs.get('variant'), context)
        if variant in self.POSSIBLE_VARIANTS:
            values['class'].append(f'nj-avatar-list--{variant}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-avatar-list {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'AvatarList': AvatarList,
    'Avatar': Avatar,
    'AvatarMore': AvatarMore,
}



"""
Sidebar
=======

See: https://www.engie.design/fluid-design-system/components/sidebar/

Sidebar can contain the entire content of the product and allows users a quick
access to a specific piece of content. The left arrow allows the user to retract
or expand the sidebar.
"""
from .base import Node

class Sidebar(Node):
    """Sidebar component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('footer',)
    "Named children."
    NODE_PROPS = ('href', 'logo_src', 'logosm_src', 'logo_width',
            'logosm_width', 'logo_height', 'logosm_height', 'logo_alt',
            'folded', 'nomotion')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['logo_width'] = self.eval(self.kwargs.get('logo_width', 100),
                context) or ''
        values['logo_height'] = self.eval(self.kwargs.get('logo_height'),
                context) or ''
        values['logosm_width'] = self.eval(self.kwargs.get('logosm_width'),
                context) or ''
        values['logosm_height'] = self.eval(
                self.kwargs.get('logosm_height', 36), context) or ''
        values['logo_src'] = self.eval(self.kwargs.get('logo_src'), context)
        values['logosm_src'] = self.eval(self.kwargs.get('logosm_src',
                values['logo_src']),
                context)
        values['logo_alt'] = self.eval(self.kwargs.get('logo_alt', ''), context)
        values['href'] = self.eval(self.kwargs.get('href', '#'), context)

        if self.eval(self.kwargs.get('folded'), context):
            values['class'].append('nj-sidebar--folded')
        if self.eval(self.kwargs.get('nomotion'), context):
            values['class'].append('nj-sidebar--no-motion')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-sidebar {class}" data-close-on-interact-out="true" {props}>
  {tmpl_logo}
  <nav class="nj-sidebar__navigation">
    <ul class="nj-list-group nj-list-group--sm nj-list-group--no-border nj-list-group--spaced-items">
      {child}
    </ul>
  </nav>
  {slot_footer}
  <ul class="nj-sidebar__collapse nj-list-group nj-list-group--sm nj-list-group--no-border">
    <li class="nj-list-group__item nj-list-group__item--clickable nj-list-group__item--no-border">
      <button data-toggle="sidebar" data-target="#{id}" aria-pressed="false">
        <span aria-hidden="true"
            class="material-icons nj-list-group__item-icon nj-sidebar__fold-btn">
          chevron_left
        </span>
        <span class="nj-list-group__item-content">Close</span>
      </button>
    </li>
  </ul>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_logo(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not values['logo_src']:
            return ''
        template = """
<a class="nj-sidebar__brand" href="{href}" title="{logo_alt}">
  <img class="nj-sidebar__logo" src="{logo_src}" alt="{logo_alt}"
      width="{logo_width}" height="{logo_height}">
  <img class="nj-sidebar__logo--folded" src="{logosm_src}" alt="{logo_alt}"
      width="{logosm_width}" height="{logosm_height}">
</a>
"""
        return self.format(template, values)


    def render_slot_footer(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<nav class="nj-sidebar__navigation nj-sidebar__navigation--footer {class}"
    {props}>
  <ul class="nj-list-group nj-list-group--sm nj-list-group--no-border nj-list-group--spaced-items">
    <div class="nj-sidebar__divider"></div>
    {child}
  </ul>
</nav>
"""
        return self.format(template, values)


class SidebarMenu(Node):
    """Sidebar menu component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('href', 'badge', 'arrow', 'current')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        context['icon_kwargs'] = {
            'class': 'nj-list-group__item-icon',
        }

        values['href'] = self.eval(self.kwargs.get('href', '#'), context)
        values['badge'] = self.eval(self.kwargs.get('badge'), context)

        if self.eval(self.kwargs.get('current'), context):
            values['class'].append('active')
            values['props'].append(('aria-current', 'true'))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-list-group__item nj-list-group__item--clickable nj-list-group__item--right-border {class}"
    {props}>
  <a href="{href}">
    {slot_icon}
    <span class="nj-list-group__item-content">{child}</span>
    {tmpl_after}
  </a>
</li>
"""
        return self.format(template, values, context)


    def render_tmpl_after(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['badge']:
            template = """
<p class="nj-badge nj-list-group__item-right-content">{badge}</p>
"""
        elif self.eval(self.kwargs.get('arrow'), context):
            template = """
<span aria-hidden="true"
    class="material-icons nj-list-group__item-icon nj-list-group__item-right-content">
  chevron_right
</span>
"""
        else:
            return ''
        return self.format(template, values)


components = {
    'Sidebar': Sidebar,
    'S_Menu': SidebarMenu,
}



"""
Navbar
======

See: https://www.engie.design/fluid-design-system/components/navbar/

The navbar helps users know where they are on the product and quickly access
other pages and features at any moment. This version is useful for application
websites with few sections.

Please check also the Header component for more possibilities.
"""
import os
import re
#-
from django.utils.translation import gettext as _
#-
from .base import COLORS, Node

class Navbar(Node):
    """Navbar component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('collapsible', 'simple')
    "Available variants."
    SLOTS = ('after',)
    "Named children."
    NODE_PROPS = ('id', 'href', 'logo_src', 'logo_alt', 'logo_width',
        'logo_height', 'expand', 'transparent', 'size', 'color')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'nav'
    "Rendered HTML tag."
    CLASS_AND_PROPS = ('logo',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_EXPANDS = ('xl',)
    "Possible values for expand argument."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-navbar--{size}')

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-navbar--{color}')

        if self.eval(self.kwargs.get('transparent'), context):
            values['class'].append('nj-navbar--transparent')
        else:
            values['class'].append('nj-navbar--shadow')

        expand = self.eval(self.kwargs.get('expand'), context)
        if expand in self.POSSIBLE_EXPANDS:
            values['class'].append(f'nj-navbar--expand-{expand}')
        elif expand is True:
            values['class'].append('nj-navbar--expand')


    def render_collapsible(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-navbar {class}" {props}>
  {tmpl_logo}
  <button class="nj-navbar__toggler" type="button" data-toggle="collapse"
      data-target="#{id}">
    <span class="nj-navbar__toggler-icon material-icons">menu</span>
  </button>
  <div class="nj-navbar--collapse nj-collapse" id="{id}">
    <ul class="nj-navbar__nav">
      {child}
    </ul>
    {slot_after}
  </div>
</{astag}>
"""
        return self.format(template, values, context)


    def render_simple(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-navbar {class}" {props}>
  {tmpl_logo}
  <ul class="nj-navbar__nav">
    {child}
  </ul>
  {slot_after}
</{astag}>
"""
        return self.format(template, values, context)


    def render_tmpl_logo(self, values, context):
        """Dynamically render a part of the component's template
        """
        logo_src = self.eval(self.kwargs.get('logo_src'), context)
        if not logo_src:
            return ''
        logo_alt = self.eval(self.kwargs.get('logo_alt', 'home'), context)
        logo_width = self.eval(self.kwargs.get('logo_width', ''), context)
        logo_height = self.eval(self.kwargs.get('logo_height', ''), context)

        _, fileext = os.path.splitext(logo_src)
        fileext = re.split('[#?]', fileext, 1)[0]
        if fileext == '.svg':
            tpl_image = f"""
<svg class="nj-navbar__logo" aria-label="{logo_alt}">
  <use href="{logo_src}" />
</svg>
"""
        else:
            tpl_image = f"""
<img class="nj-navbar__logo" src="{logo_src}" alt="{logo_alt}"
    width="{logo_width}" height="{logo_height}">
"""
        href = self.eval(self.kwargs.get('href', '/'), context)
        return f'<a class="nj-navbar__brand" href="{href}">{tpl_image}</a>'


class NavbarMenu(Node):
    """Navbar item component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('active', 'disabled', 'icon')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('icon'), context):
            values['class'].append('nj-navbar__nav-link--icon')

        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('active')

        if self.eval(self.kwargs.get('disabled'), context):
            values['class'].append('disabled')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-navbar__nav-item">
  <{astag} class="nj-navbar__nav-link {class}" {props}>
    {child}
  </{astag}>
</li>
"""
        return self.format(template, values, context)


class NavbarSearch(Node):
    """Navbar search form component
    """
    NODE_PROPS = ('action', 'method', 'id', 'color')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('form', 'icon')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_search'] = _("Search")
        values['txt_placeholder'] = _("Enter your query...")
        values['txt_close'] = _("Close")

        act = self.eval(self.kwargs.get('action'), context)
        if act:
            values['form_props'].append(('action', act))
        method = self.eval(self.kwargs.get('method'), context)
        if method:
            values['form_props'].append(('method', method))

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['icon_class'].append(f'nj-icon-material--{color}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<form class="nj-navbar__search nj-collapse {form_class}" id="{id}" {form_props}>
  <input class="nj-form-control nj-navbar__search-input {class}" type="text"
      id="{id}-input" placeholder="{txt_placeholder}" {props}>
  <button type="submit" class="nj-btn nj-navbar__search-button">
    {txt_search}
  </button>
  <a href="#" aria-label="{txt_close}" data-dismiss="#{id}"
      class="nj-navbar__nav-link nj-navbar__nav-link--icon nj-collapse-inline__close">
    <span aria-hidden="true" class="material-icons nj-icon-material {icon_class}"
        {icon_props}>
      close
    </span>
  </a>
</form>
"""
        return self.format(template, values)


class NavbarSearchIcon(Node):
    """Navbar search button component for navbar search form
    """
    NODE_PROPS = ('target', 'color')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('icon',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['target'] = self.eval(self.kwargs.get('target'), context)

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['icon_class'].append(f'nj-icon-material--{color}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-navbar__nav-item">
  <a class="nj-navbar__nav-link nj-navbar__nav-link--icon {class}"
      data-toggle="collapse" href="#{target}" aria-expanded="false"
      aria-controls="{target}" {props}>
    <span aria-hidden="true" class="material-icons nj-icon-material {icon_class}"
        {icon_props}>
      search
    </span>
  </a>
</li>
"""
        return self.format(template, values)


components = {
    'Navbar': Navbar,
    'N_Menu': NavbarMenu,
    'N_MenuSearch': NavbarSearchIcon,
    'N_Search': NavbarSearch,
}



"""
Stamp
=====

See: https://www.engie.design/fluid-design-system/components/stamp/

Stamp is a special brand identity component for Act with ENGIE operation
"""
from .base import Node

class Stamp(Node):
    """Stamp component
    """
    NODE_PROPS = ('id', 'gradient', 'shadow')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('circle',)
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['label_parts'] = values['label'].split(' ', 3)
        values['gradient'] = self.eval(self.kwargs.get('gradient'), context)
        if values['gradient']:
            values['circle_props'].append(('fill',
                    f"url(#{values['id']}-gradient)"))
        else:
            values['circle_props'].append(('fill', '#fff'))

        if self.eval(self.kwargs.get('shadow'), context):
            values['class'].append('nj-stamp--shadow')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<svg xmlns="http://www.w3.org/2000/svg" class="nj-stamp {class}" {props}>
  <defs>
    {tmpl_gradient}
    <mask id="{id}-mask" x="0" y="0" width="100%" height="100%">
      <circle class="nj-stamp__overlay" cx="85" cy="85" r="85"/>
      <text class="nj-stamp__text" y="67" transform="translate(85)">
        {tmpl_label1}
        {tmpl_label2}
        {tmpl_label3}
      </text>
    </mask>
  </defs>
  <circle cx="85" cy="85" r="85" mask="url(#{id}-mask)" {circle_props}/>
</svg>
"""
        return self.format(template, values, context)


    def render_tmpl_gradient(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if values['gradient']:
            stops = []
            grads = [x for x in values['gradient'].split(' ') if x.strip()]
            for ii, grad in enumerate(grads):
                pos = int(ii / (len(grads) - 1) * 100)
                stops.append(f'<stop offset="{pos}%" stop-color="{grad}"/>')
            values['stops'] = '\n'.join(stops)
            template = """
<linearGradient id="{id}-gradient" x1="0" x2="1" y1="0" y2="1">
  {stops}
</linearGradient>
"""
            return self.format(template, values)
        return ''


    def render_tmpl_label1(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if len(values['label_parts']) > 0:
            label = values['label_parts'][0]
            return f'<tspan x="0" text-anchor="middle">{label}</tspan>'
        return ''


    def render_tmpl_label2(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if len(values['label_parts']) > 1:
            label = values['label_parts'][1]
            return f'<tspan x="0" text-anchor="middle" dy="28">{label}</tspan>'
        return ''


    def render_tmpl_label3(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if len(values['label_parts']) > 2:
            label = values['label_parts'][2]
            return f'<tspan x="0" text-anchor="middle" dy="28">{label}</tspan>'
        return ''


components = {
    'Stamp': Stamp,
}



"""
Breadcrumb
==========

See: https://www.engie.design/fluid-design-system/components/breadcrumb/

Breadcrumbs should be used as soon as you structure information hierarchically.
Breadcrumbs provide users with their current location, help them find related
content and serve as secondary navigation.
""" # pylint:disable=line-too-long

from django.utils.translation import gettext as _
#-
from .base import Node

class Breadcrumb(Node):
    """Breadcrumb component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'nav'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_breadcrumb'] = _("breadcrumb")
        tag = self.eval(self.kwargs.get('astag'), context)
        if tag and tag != 'nav':
            values['props'].append(('role', 'navigation'))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} aria-label="{txt_breadcrumb}" {props}>
  <ol class="nj-breadcrumb">
    {child}
  </ol>
</{astag}>
"""
        return self.format(template, values)


class BreadcrumbItem(Node):
    """Breadcrumb item
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('href', 'current', 'isicon')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('link',)
    "Prepare xxx_class and xxx_props values."

    is_current = False

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['href'] = self.eval(self.kwargs.get('href', '#'), context)
        self.is_current = self.eval(self.kwargs.get('current', False), context)
        if self.is_current:
            values['props'].append(('aria-current', 'page'))

        if self.eval(self.kwargs.get('isicon', False), context):
            values['link_class'].append('nj-link-icon')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-breadcrumb__item {class}" {props}>{tmpl_item}</li>
"""
        return self.format(template, values, context)


    def render_tmpl_item(self, values, context):
        """Dynamically render a part of the component's template
        """
        if self.is_current:
            return values['child']

        template = """
<a href="{href}" class="nj-link nj-link--sm nj-link--grayed {link_class}"
    {link_props}>
  {child}
</a>
"""
        return self.format(template, values)


components = {
    'Breadcrumb': Breadcrumb,
    'BreadcrumbItem': BreadcrumbItem,
}



"""
Inline Message
==============

See: https://www.engie.design/fluid-design-system/components/inline-message/
"""
from django.utils.translation import gettext as _
#-
from . base import Node

class InlineMessage(Node):
    """Inline message component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('title',)
    "Named children."
    NODE_PROPS = ('style', 'icon', 'close')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('icon', 'close')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_STYLES = ('error', 'info', 'success', 'warning', 'fatal-error')
    "Possible values for style argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_close'] = _("Hide message")
        style = self.eval(self.kwargs.get('style'), context)
        if style != 'error':
            values['class'].append(f'nj-inline-message--{style}')
        if style == 'fatal-error':
            values['close_class'].append('nj-icon-btn--inverse')
        if style in self.POSSIBLE_STYLES:
            values['icon_class'].append(f'nj-status-indicator--{style}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-inline-message {class}" {props}>
  {tmpl_icon}
  <div class="nj-inline-message__content">
   <h4 class="nj-inline-message__title">{slot_title}</h4>
   <p class="nj-inline-message__body">{child}</p>
  </div>
  {tmpl_close}
</{astag}>
"""
        return self.format(template, values, context)


    def render_tmpl_icon(self, values, context):
        """Dynamically render a part of the component's template.
        """
        show_icon = self.eval(self.kwargs.get('icon', True), context)
        if not show_icon:
            return ''
        style = self.eval(self.kwargs.get('style'), context)
        if style == 'fatal-error':
            return ''
        tmpl = """
<div class="nj-inline-message__status nj-status-indicator {icon_class}"
    aria-hidden="true" {icon_props}>
  <div class="nj-status-indicator__svg"></div>
</div>
"""
        return tmpl.format(**values)


    def render_tmpl_close(self, values, context):
        """Dynamically render a part of the component's template.
        """
        show_close = self.eval(self.kwargs.get('close'), context)
        if not show_close:
            return ''
        tmpl = """
<button class="nj-inline-message__close nj-icon-btn {close_class}"
    {close_props}>
  <span class="nj-sr-only">{txt_close}</span>
  <span aria-hidden="true" class="nj-icon-btn__icon material-icons">close</span>
</button>
"""
        return tmpl.format(**values)


components = {
    'Message': InlineMessage,
}



"""
Table
=====

See: https://www.engie.design/fluid-design-system/components/table/

Data table is used to display and organise all the data set. A data table is
used to compare and analyze data sets.The data informations are always
displayed in row and column.
"""
from .base import Node

class Table(Node):
    """Table component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('style',)
    "Extended Template Tag arguments."
    POSSIBLE_STYLES = ('default', 'striped', 'hover')
    "Possible values for style argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        style = self.eval(self.kwargs.get('style'), context)
        if style in self.POSSIBLE_STYLES[1:]:
            values['class'].append(f'nj-table--{style}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<table class="nj-table {class}" {props}>
  {tmpl_label}
  {child}
</table>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['label']:
            return self.format('<caption>{label}</caption>', values)
        return ''


components = {
    'Table': Table,
}



"""
Toggle
======

See: https://www.engie.design/fluid-design-system/components/toggle/

Use toggles when your users can turn something on or off instantly. For example,
if they need to enable or disabled the notifications. This component have been
in user interfaces for a long time so it should be used as expected. Keep in
mind that toggle should only be used when the user needs to decide between two
opposing states.

The toggle is an important element of the interface because the user often has
an interaction with it. This is why it must be visible at a glance if a switch
has been selected.
"""
from .base import FormNode
from .base_widgets import CustomCheckboxInput

class Toggle(FormNode):
    """Toggle component
    """
    NODE_PROPS = ('size', 'icon', 'darkmode')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'wrapper')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_SIZES = ('lg',)
    "Possible values for size argument."
    widget_class = CustomCheckboxInput

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['wrapper_class'].append(f'nj-toggle--{size}')

        if self.eval(self.kwargs.get('disabled'), context):
            values['wrapper_class'].append('nj-toggle--disabled')

        values['darkmode'] = self.eval(self.kwargs.get('darkmode'), context)
        values['icon'] = self.eval(self.kwargs.get('icon'), context)
        if values['darkmode'] or values['icon']:
            values['wrapper_props'].append(('title', values['label']))


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['role'] = 'switch'


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-toggle {wrapper_class}" {wrapper_props}>
  <label for="{id}">
    {tmpl_element}
    <span class="nj-toggle__track"></span>
    {tmpl_icon}
    {tmpl_label}
  </label>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if values['icon'] or values['darkmode']:
            tmpl = '<span class="nj-sr-only">{label}</span>'
        else:
            tmpl = '<span">{label}</span>'
        return self.format(tmpl, values)


    def render_tmpl_icon(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if values['darkmode']:
            return """
<svg aria-hidden="true" focusable="false" xmlns="http://www.w3.org/2000/svg"
    viewBox="0 0 16 16" class="nj-toggle__icon">
  <path d="M9.53 16h-.48a8 8 0 01-7.4-8.47A7.94 7.94 0 019.52 0h.94l-.64.6a7.71 7.71 0 00-2.06 8 7.6 7.6 0 006.3 5.23l.87.1-.7.52A7.77 7.77 0 019.53 16zM8.71.74a7.31 7.31 0 00.38 14.54 7.06 7.06 0 004-.94 8.35 8.35 0 01-6-5.55A8.48 8.48 0 018.71.74z"/>
</svg>
"""
        if values['icon']:
            tmpl = """
<span aria-hidden="true" class="nj-toggle__icon material-icons">{icon}</span>
"""
            return self.format(tmpl, values)
        return ''


components = {
    'Toggle': Toggle,
}



"""
Grid
====

See: https://www.engie.design/fluid-design-system/components/grid/

All about the grid! Grid systems are used to create perfect layouts. Our grid
system is based on the Bootstrap v4 grid. Use the mobile-first flexbox grid to
build layouts of all shapes and sizes with a twelve column system, five default
responsive tiers, Sass variables and mixins, and dozens of predefined classes.
"""
from .base import Node

class Grid(Node):
    """Grid component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'div'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="container {class}" {props}>{child}</{astag}>
"""
        return self.format(template, values)


class GridRow(Node):
    """Grid row component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    COL_SIZES = ('col', 'sm', 'md', 'lg')
    "Column sizes."
    NODE_PROPS = (
            'gutter',
            *['align_%s' % x for x in COL_SIZES],
            *['valign_%s' % x for x in COL_SIZES])
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'div'
    "Rendered HTML tag."
    POSSIBLE_ALIGNS = ('start', 'center', 'end', 'around', 'between')
    "Possible values for align argument."
    POSSIBLE_VALIGNS = ('start', 'center', 'end')
    "Possible values for valign argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        for size in self.COL_SIZES:
            align = self.eval(self.kwargs.get('align_%s' % size), context)

            if size == 'col':
                prefix = 'justify-content'
            else:
                prefix = f'justify-content-{size}'

            if align in self.POSSIBLE_ALIGNS:
                values['class'].append(f'{prefix}-{align}')

            valign = self.eval(self.kwargs.get('valign_%s' % size), context)

            if size == 'col':
                prefix = 'align-items'
            else:
                prefix = f'align-items-{size}'

            if valign in self.POSSIBLE_VALIGNS:
                values['class'].append(f'{prefix}-{valign}')

        if not self.eval(self.kwargs.get('gutter', True), context):
            values['class'].append('no-gutters')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = '<{astag} class="row {class}" {props}>{child}</{astag}>'
        return self.format(template, values)


class GridColumn(Node):
    """Grid column component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    COL_SIZES = ('col', 'sm', 'md', 'lg', 'xl')
    "Column sizes."
    NODE_PROPS = (
            'order',
            *COL_SIZES,
            *['offset_%s' % x for x in COL_SIZES],
            *['ml_%s' % x for x in COL_SIZES],
            *['mr_%s' % x for x in COL_SIZES])
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'div'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        for size in self.COL_SIZES:
            width = self.eval(self.kwargs.get(size), context)
            try:
                if width not in ('fill', 'shrink'):
                    width = int(width)
            except (ValueError, TypeError):
                width = None

            if size == 'col':
                prefix = 'col'
            else:
                prefix = f'col-{size}'

            if width == 'fill':
                values['class'].append(prefix)
            elif width == 'shrink':
                values['class'].append(f'{prefix}-auto')
            elif width:
                values['class'].append(f'{prefix}-{width}')

            offset = self.eval(self.kwargs.get(f'offset_{size}'), context)
            try:
                offset = int(offset)
            except (ValueError, TypeError):
                offset = None

            if offset:
                if size == 'col':
                    prefix = 'offset'
                else:
                    prefix = f'offset-{size}'

                values['class'].append(f'{prefix}-{offset}')

            margin_left = self.eval(self.kwargs.get(f'ml_{size}'), context)
            try:
                if margin_left not in ('auto',):
                    margin_left = int(margin_left)
            except (ValueError, TypeError):
                margin_left = None

            if margin_left:
                if size == 'col':
                    prefix = 'ml'
                else:
                    prefix = f'ml-{size}'

                values['class'].append(f'{prefix}-{margin_left}')

            margin_right = self.eval(self.kwargs.get(f'mr_{size}'), context)
            try:
                if margin_right not in ('auto',):
                    margin_right = int(margin_right)
            except (ValueError, TypeError):
                margin_right = None

            if margin_right:
                if size == 'col':
                    prefix = 'mr'
                else:
                    prefix = f'mr-{size}'

                values['class'].append(f'{prefix}-{margin_right}')

        order = self.eval(self.kwargs.get('order'), context)
        try:
            order = int(order)
            values['class'].append(f'order-{order}')
        except (ValueError, TypeError):
            pass


    def render_default(self, values, context):
        """Html output of the component
        """
        template = '<{astag} class="{class}" {props}>{child}</{astag}>'
        return self.format(template, values)


components = {
    'Grid': Grid,
    'Row': GridRow,
    'Col': GridColumn,
}



"""
Toast
=====

See: https://www.engie.design/fluid-design-system/components/toast/

Toasts are non-modal dialogs used as a way to provide feedback following user
action. They are typically composed of a short message appearing at the bottom
of the screen, to make them as discreet as possible.
"""
from django.utils.translation import gettext as _
#-
from .base import COLORS, Node

class Toast(Node):
    """Toast component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('content_id', 'color', 'btncolor', 'gauge')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('button', 'gauge')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_COLORS = COLORS
    "Possible values for color or btncolor argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        context['icon_icon_kwargs'] = {
            'class': 'nj-toast__icon',
        }

        values['txt_close'] = _("Close notification")
        values['txt_gauge'] = _("The toast will be automatically closed in "
                "%(gauge)ss")

        values['content_id'] = self.eval(self.kwargs.get('content_id'), context)

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-toast--{color}')

        color = self.eval(self.kwargs.get('btncolor'), context)
        if color in self.POSSIBLE_COLORS:
            values['button_class'].append(f'nj-icon-btn--{color}')

        values['gauge'] = gauge = self.eval(self.kwargs.get('gauge'), context)
        if gauge:
            values['txt_gauge'] = values['txt_gauge'] % {'gauge': gauge}
            if gauge != 5:
                values['gauge_props'].append(
                        ('style', f'animation-duration: {gauge}s;'))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-toast {class}" {props}>
  <div class="nj-toast__body">
    {slot_icon}
    <div class="nj-toast__content">
      {child}
    </div>
  </div>
  {tmpl_closebtn}
  {tmpl_gauge}
</{astag}>
"""
        return self.format(template, values, context)


    def render_tmpl_closebtn(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['content_id']:
            template = """
<div class="nj-toast__action">
  <button type="button" class="nj-icon-btn nj-icon-btn--lg {button_class}"
      aria-describedby="{content_id}" {button_props}>
    <span class="nj-sr-only">{txt_close}</span>
    <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
      close
    </span>
  </button>
</div>
"""
            return self.format(template, values)
        return ''


    def render_tmpl_gauge(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['gauge']:
            template = """
<div class="nj-toast__gauge">
  <div class="nj-toast__gauge-bar {gauge_class}" {gauge_props}>
    <p class="nj-sr-only">{txt_gauge}</p>
  </div>
</div>
"""
            return self.format(template, values)
        return ''



class ToastText(Node):
    """Toast text component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'p'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-toast__text {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


class ToastTitle(Node):
    """Toast title component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'p'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-toast__title {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


class ToastContainer(Node):
    """Toast component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('fullwidth',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if values['label']:
            values['props'].append(('aria-label', values['label']))

        if self.eval(self.kwargs.get('fullwidth'), context):
            values['class'].append('nj-toast__container--full-width')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-toast__container {class}" role="region" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Toast': Toast,
    'ToastText': ToastText,
    'ToastTitle': ToastTitle,
    'ToastContainer': ToastContainer,
}



"""
Slider
======

See: https://www.engie.design/fluid-design-system/components/slider/

A slider allows the user to slide a knob along a straight track to control or to
change a variable. Sliders can have icons on both ends of the bar, for example
when you want to change the brightness of your screen.
"""
from .base import FormNode
from .base_widgets import CustomRangeInput

class Slider(FormNode):
    """Slider component
    """
    NODE_PROPS = ('tooltip',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'wrapper')
    "Prepare xxx_class and xxx_props values."
    widget_class = CustomRangeInput

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('disabled'), context):
            values['wrapper_class'].append('nj-slider--disabled')
        if self.eval(self.kwargs.get('tooltip'), context):
            values['wrapper_props'].append(('data-tooltip', 'true'))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-slider {wrapper_class}" {wrapper_props}>
  {tmpl_label}
  {tmpl_element}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values['label']:
            return ''
        if self.bound_field.field.required or\
                self.eval(self.kwargs.get('required'), context):
            tmpl = """
<label for="{id}" class="{label_class}" {label_props}>
  {label}{label_suffix}
  <span class="nj-form-item__required-asterisk">*</span>
</label>
"""
        else:
            tmpl = """
<label for="{id}" class="{label_class}" {label_props}>
  {label}{label_suffix}
</label>
"""
        return self.format(tmpl, values)


components = {
    'Slider': Slider,
}



"""Template tags provided by fluid_design module.

Provides:
 - svg for loading svg file
 - fluid_design_assets for loading css/js files
 - built-in template tags (you can make your own) for Fluid Design System
   components

"""
import logging
import os
import re
#-
from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
#-
from ..tags import (autocomplete, avatar, badge, base, breadcrumb, button,
        card, checkbox_set, collapse, fab, footer, form_item, grid, header,
        icon, icon_button, inline_message, link, list_, modal, navbar,
        pagination, progress, radio_set, segmented_control, sidebar, skeleton,
        slider, spinner, stamp, status_indicator, table, tabs, tag, toast,
        toggle)

_logger = logging.getLogger(__name__)
register = template.Library()

FLUID_DESIGN_TAGS = {
    **autocomplete.components,
    **avatar.components,
    **badge.components,
    **base.components,
    **breadcrumb.components,
    **button.components,
    **card.components,
    **checkbox_set.components,
    **collapse.components,
    **fab.components,
    **footer.components,
    **form_item.components,
    **grid.components,
    **header.components,
    **icon.components,
    **icon_button.components,
    **inline_message.components,
    **link.components,
    **list_.components,
    **modal.components,
    **navbar.components,
    **pagination.components,
    **progress.components,
    **radio_set.components,
    **segmented_control.components,
    **sidebar.components,
    **skeleton.components,
    **slider.components,
    **spinner.components,
    **stamp.components,
    **status_indicator.components,
    **table.components,
    **tabs.components,
    **tag.components,
    **toast.components,
    **toggle.components,
}


class TagParser:
    """Parse arguments of Django template tags.
    """
    def __init__(self, tags):
        self.tags = tags

    def __call__(self, parser, token):
        params = token.split_contents()
        tagname = params.pop(0)

        args = []
        kwargs = {}
        for param in params:
            if '=' in param:
                key, val = param.split('=', 1)
            else:
                key, val = (None, param)

            if val[0] in ('"', "'"):
                val = val.strip('"\'')
            else:
                val = base.VariableInVariable(val, parser)

            if key:
                kwargs[key] = val
            else:
                args.append(val)

        cls = self.tags[tagname]

        if getattr(cls, 'WANT_CHILDREN', False):
            nodelist = parser.parse(('end' + tagname,))
            parser.delete_first_token()
            args.insert(0, nodelist)

        return cls(*args, parser=parser, **kwargs)


@register.simple_tag
def svg(path):
    """This template tag will load svg files located in settings.SVG_DIRS.
    """
    # Security test.
    if not path or not re.match(r'^\w.*\.svg$', path):
        return ''
    for dirname in settings.SVG_DIRS:
        if not os.path.exists(dirname/path):
            continue
        return mark_safe(open(dirname/path, 'r', encoding='utf-8').read())
    return ''


@register.simple_tag
def fluid_design_assets():
    """This template tag will inject Fluid Design System css/js files.

    Don't forget to run collectstatic!
    """
    if settings.DEBUG:
        suffix = ''
    else:
        suffix = '' # already minified
    tmpl = """
<link rel="stylesheet" type="text/css" href="{baseurl}fluid_design/fluid-design-system{suffix}.css">
<script src="{baseurl}fluid_design/fluid-design-system{suffix}.js"></script>
""" # pylint:disable=line-too-long
    return mark_safe(tmpl.format(baseurl=settings.STATIC_URL, suffix=suffix))


_parser = TagParser(FLUID_DESIGN_TAGS)
for name in FLUID_DESIGN_TAGS:
    register.tag(name, _parser)



"""
List
====

See: https://www.engie.design/fluid-design-system/components/list/

Lists are a flexible and powerful component for displaying a series of content.
"""
from .base import Node

class List(Node):
    """List component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('size', 'border', 'spaced')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'ul'
    "Rendered HTML tag."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if not self.eval(self.kwargs.get('border', True), context):
            values['class'].append('nj-list-group--no-border')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-list-group--{size}')

        if self.eval(self.kwargs.get('spaced'), context):
            values['class'].append('nj-list-group--spaced-items')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-list-group {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


class ListItem(Node):
    """List component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon_first', 'icon_last')
    "Named children."
    NODE_PROPS = ('active', 'border', 'click', 'disabled')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'span'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        context['icon_kwargs'] = {
            'class': 'nj-list-group__item-icon',
        }
        context['icon_last_icon_kwargs'] = {
            'class': 'nj-list-group__item-right-content',
        }
        context['icon_last_badge_kwargs'] = {
            'class': 'nj-list-group__item-right-content',
        }
        context['icon_last_tag_kwargs'] = {
            'class': 'nj-list-group__item-right-content',
        }

        if self.eval(self.kwargs.get('click'), context):
            values['class'].append('nj-list-group__item--clickable')

        if self.eval(self.kwargs.get('border'), context):
            values['class'].append('nj-list-group__item--right-border')

        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('active')
            values['props'].append(('aria-current', 'true'))

        if self.eval(self.kwargs.get('disabled'), context):
            values['class'].append('disabled')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-list-group__item {class}" {props}>
  {slot_icon_first}
  {tmpl_content}
  {slot_icon_last}
</li>
"""
        return self.format(template, values, context)


    def render_tmpl_content(self, values, context):
        """Dynamically render a part of the component's template
        """
        if 'icon_last' in self.slots:
            tmpl = """
<{astag} class="nj-list-group__item-content">
  {child}
</{astag}>
"""
            return self.format(tmpl, values)
        return values['child']


components = {
    'List': List,
    'Li': ListItem,
}



"""
Skeleton
========

See: https://www.engie.design/fluid-design-system/components/skeleton/

Show a preview placeholder of your content to reduce load-time frustration.
"""
from django.utils.translation import gettext as _
#-
from .base import Node

class Skeleton(Node):
    """Skeleton component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if not values['label']:
            values['label'] = _("Loading...")


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-skeleton-container {class}" {props}>
  <span class="nj-sr-only">{label}</span>
  {child}
</div>
"""
        return self.format(template, values, context)


class SkeletonItem(Node):
    """Skeleton component
    """
    NODE_PROPS = ('shape', 'height', 'size')
    "Extended Template Tag arguments."
    POSSIBLE_SHAPES = ('icon', 'svg', 'circle', 'rectangle')
    "Possible values for shape argument."
    POSSIBLE_HEIGHTS = ('peta', 'tera', 'giga', 'mega', 'kilo', 'hecto', 'deca',
            'base', 'deci', 'centi')
    "Possible values for height argument."
    POSSIBLE_SIZES = ('sm', 'base', 'lg', 'xl')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        shape = self.eval(self.kwargs.get('shape'), context)
        if shape in self.POSSIBLE_SHAPES:
            if shape in ('icon', 'svg'):
                values['class'].append('nj-skeleton--area')
            else:
                values['class'].append(f'nj-skeleton--{shape}')
        values['shape'] = shape

        height = self.eval(self.kwargs.get('height'), context)
        if height in self.POSSIBLE_HEIGHTS:
            values['class'].append(f'nj-skeleton--{height}')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-skeleton--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div aria-hidden="true" class="nj-skeleton {class}" {props}>
  {tmpl_child}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_child(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['shape'] == 'icon':
            return """
<span class="nj-skeleton__icon material-icons nj-icon-material nj-icon-material--tertiary nj-icon-material--xl">
  image
</span>
"""
        if values['shape'] == 'svg':
            return """
<svg class="nj-skeleton__icon" width="48" height="48" viewBox="0 0 36 36"
    fill="none" xmlns="http://www.w3.org/2000/svg">
  <path
    d="M36 32V4C36 1.8 34.2 0 32 0H4C1.8 0 0 1.8 0 4V32C0 34.2 1.8 36 4 36H32C34.2 36 36 34.2 36 32ZM11 21L16 27.02L23 18L32 30H4L11 21Z"
    fill="currentColor" />
</svg>
"""
        return ''


components = {
    'Skeleton': Skeleton,
    'Bone': SkeletonItem,
}



"""Helper functions for working with datetime format strings

Django uses PHP format in templates and Python format everywhere else.
"""
from datetime import datetime
#-
from django.utils.formats import get_format
import pytz

SAMPLE_DATETIME = datetime(1979, 5, 23, 13, 59, 43, 123,
        pytz.timezone('Asia/Jakarta'))

# From Flatpickr (which is used by Carbon Design)
TOKEN_REGEX = {
    'D': "(\\w+)",
    'F': "(\\w+)",
    'G': "(\\d\\d|\\d)",
    'H': "(\\d\\d|\\d)",
    'J': "(\\d\\d|\\d)\\w+",
    # ToDo: K is ignored
    'M': "(\\w+)",
    'S': "(\\d\\d|\\d)",
    'U': "(.+)",
    'W': "(\\d\\d|\\d)",
    'Y': "(\\d\\d\\d\\d)",
    'Z': "(.+)",
    'd': "(\\d\\d|\\d)",
    'h': "(\\d\\d|\\d)",
    'i': "(\\d\\d|\\d)",
    'j': "(\\d\\d|\\d)",
    'l': "(\\w+)",
    'm': "(\\d\\d|\\d)",
    'n': "(\\d\\d|\\d)",
    's': "(\\d\\d|\\d)",
    'u': "(.+)",
    'w': "(\\d\\d|\\d)",
    'y': "(\\d\\d)",
}


def dateformat_to_pattern(value):
    """Calculate the value of HTML pattern attribute
    """
    return ''.join(TOKEN_REGEX.get(x, x) for x in value)


def dateformat_python_to_php(python_format_string):
    """Convert datetime format string from python to php

    Given a python datetime format string, attempts to convert it to the nearest
    PHP datetime format string possible. 

    See: https://djangosnippets.org/snippets/10563/
    """
    python2PHP = {
            "%a": "D", "%A": "l", "%b": "M", "%B": "F", "%c": "", "%d": "d",
            "%H": "H", "%I": "h", "%j": "z", "%m": "m", "%M": "i", "%p": "A",
            "%S": "s", "%U": "", "%w": "w", "%W": "W", "%x": "", "%X": "",
            "%y": "y", "%Y": "Y", "%Z": "e" }

    php_format_string = python_format_string
    for py, php in python2PHP.items():
        php_format_string = php_format_string.replace(py, php)

    return php_format_string


def get_field_dateformat(bound_field):
    """Calculate bound field date format

    ToDo: This is oversimplified, I think I missed some things.
    """
    field = bound_field.field
    widget = field.widget

    if getattr(widget, 'format', None):
        fmt = widget.format
    elif getattr(widget, 'format_key', None):
        fmt = get_format(widget.format_key)[0]
    elif getattr(field, 'input_formats', None):
        fmt = get_format(field.input_formats[0])[0]
    else:
        return None

    return dateformat_python_to_php(fmt)



"""
Button
======

See: https://www.engie.design/fluid-design-system/components/button/

Buttons allow users to interact with the product and trigger actions. They can
be of different sizes, colors and status.
In terms of accessibility, be mindful of people using assistive technologies:
don’t use links instead of buttons to trigger actions.
"""

from .base import COLORS, Node

class Button(Node):
    """Button component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('disabled', 'variant', 'color', 'size')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."
    POSSIBLE_VARIANTS = ('subtle', 'minimal')
    "Possible values for variant argument."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    POSSIBLE_SIZES = ('xs', 'sm', 'lg')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('disabled'), context):
            values['props'].append(('disabled', ''))

        variant = self.eval(self.kwargs.get('variant'), context)
        if variant in self.POSSIBLE_VARIANTS:
            values['class'].append(f'nj-btn--{variant}')

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-btn--{color}')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-btn--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-btn {class}" {props}>
  {slot_icon}
  {child}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Button': Button,
}



"""
Progress bar
============

See: https://www.engie.design/fluid-design-system/components/progress/

Progress bars allow users to know their task was successfully launched and the
system progresses towards task completion. They are a representation of a
progress status that evolves over time. As a general rule of thumb, use progress
bars when task completion takes longer than 1 second.
"""
from .base import Node

class ProgressBar(Node):
    """Progress bar component
    """
    NODE_PROPS = ('current', 'min', 'max', 'text')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if values['label']:
            values['props'].append(('aria-label', values['label']))

        current = self.eval(self.kwargs.get('current', 0), context)
        value_min = self.eval(self.kwargs.get('min', 0), context)
        value_max = self.eval(self.kwargs.get('max', 100), context)

        percent = current * 100 / (value_max - value_min)
        if int(percent) == percent:
            percent = int(percent)
        else:
            percent = round(percent, 2)
        values['percent'] = percent

        values['text'] = self.eval(self.kwargs.get('text'), context)

        if percent:
            values['props'].append(('style', f'width: {percent}%'))
        values['props'].append(('aria-valuenow', current))
        values['props'].append(('aria-valuemin', value_min))
        values['props'].append(('aria-valuemax', value_max))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-progress">
  <div class="nj-progress__bar {class}" role="progressbar" {props}>
    <span class="nj-sr-only">{percent}%</span>
  </div>
  {tmpl_text}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_text(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not values['text']:
            return ''

        tpl = '<div aria-hidden="true" class="nj-progress__text">{text}</div>'
        return tpl.format(text=values['text'])


components = {
    'Progress': ProgressBar,
}



"""
Card
====

See: https://www.engie.design/fluid-design-system/components/card/

Cards help bring hierarchy and visual consistency to the information displayed
on a page, especially when the content is heterogenous. They are excellent ways
to display rich media content like images or videos or to highlight
action-required elements.
"""
from .base import Node

class Card(Node):
    """Card component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('date', 'details', 'description', 'growth', 'header', 'image',
            'number', 'price', 'subtitle', 'title')
    "Named children."
    MODES = ('default', 'cover')
    "Available variants."
    NODE_PROPS = ('border', 'variant', 'align')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('body',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_VARIANTS = ('horizontal',)
    "Possible values for variant argument."
    POSSIBLE_ALIGNS = ('center',)
    "Possible values for align argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('border'), context):
            values['class'].append('nj-card--border')

        variant = self.eval(self.kwargs.get('variant'), context)
        if variant in self.POSSIBLE_VARIANTS:
            values['class'].append(f'nj-card--{variant}')

        align = self.eval(self.kwargs.get('align'), context)
        if align in self.POSSIBLE_ALIGNS:
            values['body_class'].append(f'text-{align}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-card {class}" {props}>
  {slot_image}
  {slot_header}
  <div class="nj-card__body {body_class}" {body_props}>
    {slot_details}
    {slot_title}
    {slot_price}
    {child}
    {slot_number}
    {slot_growth}
    {slot_subtitle}
    {slot_date}
  </div>
</{astag}>
"""
        return self.format(template, values, context)


    def render_cover(self, values, context):
        """Html output of the component
        """
        template = """
<a class="nj-card nj-card--cover {class}" {props}>
  <div class="nj-card__body {body_class}" {body_props}>
    <div class="nj-card__overlay">
      {slot_title}
      <span class="material-icons" aria-hidden="true">arrow_forward</span>
      {slot_description}
    </div>
  </div>
</a>
"""
        return self.format(template, values, context)


    def render_slot_title(self, values, context):
        """Render html of the slot.
        """
        if not values['astag']:
            values['astag'] = 'h4'
        tmpl = """
<{astag} class="nj-card__title {class}" {props}>
  {child}
</{astag}>
"""
        return tmpl.format(**values)


    def render_slot_subtitle(self, values, context):
        """Render html of the slot.
        """
        if not values['astag']:
            values['astag'] = 'h4'
        tmpl = """
<{astag} class="nj-card__subtitle {class}" {props}>
  {child}
</{astag}>
"""
        return tmpl.format(**values)


    def render_slot_header(self, values, context):
        """Render html of the slot.
        """
        tmpl = '<div class="nj-card__header {class}" {props}>{child}</div>'
        return tmpl.format(**values)


    def render_slot_description(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<p class="nj-card__description {class}" {props}>
  {child}
</p>
"""
        return tmpl.format(**values)


    def render_slot_details(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<p class="nj-card__details {class}" {props}>
  {child}
</p>
"""
        return tmpl.format(**values)


    def render_slot_image(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<div class="nj-card__img-wrapper {class}" {props}>
  {child}
</div>
"""
        return tmpl.format(**values)


    def render_slot_price(self, values, context):
        """Render html of the slot.
        """
        tmpl = '<div class="nj-card__price {class}" {props}>{child}</div>'
        return tmpl.format(**values)


    def render_slot_number(self, values, context):
        """Render html of the slot.
        """
        tmpl = '<p class="nj-card__number {class}" {props}>{child}</p>'
        return tmpl.format(**values)


    def render_slot_growth(self, values, context):
        """Render html of the slot.
        """
        tmpl = '<p class="nj-card__growth {class}" {props}>{child}</p>'
        return tmpl.format(**values)


    def render_slot_date(self, values, context):
        """Render html of the slot.
        """
        tmpl = '<p class="nj-card__date {class}" {props}>{child}</p>'
        return tmpl.format(**values)


class CardImage(Node):
    """Card img component
    """
    def render_default(self, values, context):
        """Html output of the component
        """
        template = '<img class="nj-card__img {class}" {props}>'
        return self.format(template, values)


class CardList(Node):
    """Card list component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('variant',)
    "Extended Template Tag arguments."
    POSSIBLE_VARIANTS = ('columns', 'deck')
    "Possible values for variant argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        variant = self.eval(self.kwargs.get('variant'), context)
        if variant in self.POSSIBLE_VARIANTS:
            values['class'].append(f'nj-card-{variant}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = '<div class="{class}" {props}>{child}</div>'
        return self.format(template, values)


components = {
    'Card': Card,
    'CardList': CardList,
    'CardImage': CardImage,
}



"""
Header
======

See: https://www.engie.design/fluid-design-system/components/header/

The header is a structuring element of ENGIE's identity. It is the main
navigation of a website. This version is primarily intended for the corporate
website with many sections.
Please check also the Navbar component for a more compact version.
"""
from django.utils.translation import gettext as _
#-
from .base import COLORS, Node

class Header(Node):
    """Header component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('head_first', 'head_last', 'search')
    "Named children."
    NODE_PROPS = ('fixed', 'size', 'scroll', 'expand', 'href', 'logo_src',
        'logosm_src', 'logo_width', 'logosm_width', 'logo_height',
        'logosm_height', 'logo_alt')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'header'
    "Rendered HTML tag."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."
    POSSIBLE_SCROLLS = ('sm',)
    "Possible values for scroll argument."
    POSSIBLE_EXPANDS = ('lg',)
    "Possible values for expand argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['logo_width'] = self.eval(self.kwargs.get('logo_width'),
                context) or ''
        values['logo_height'] = self.eval(self.kwargs.get('logo_height', 48),
                context) or ''
        values['logosm_width'] = self.eval(self.kwargs.get('logosm_width'),
                context) or ''
        values['logosm_height'] = self.eval(
                self.kwargs.get('logosm_height', 32), context) or ''
        values['logo_src'] = self.eval(self.kwargs.get('logo_src'), context)
        values['logosm_src'] = self.eval(self.kwargs.get('logosm_src',
                values['logo_src']),
                context)
        values['logo_alt'] = self.eval(self.kwargs.get('logo_alt', ''), context)
        values['href'] = self.eval(self.kwargs.get('href', '#'), context)

        if self.eval(self.kwargs.get('fixed'), context):
            values['class'].append('nj-header--fixed')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SCROLLS:
            values['class'].append(f'nj-header--{size}')

        scroll = self.eval(self.kwargs.get('scroll'), context)
        if scroll in self.POSSIBLE_SCROLLS:
            values['class'].append(f'nj-header--scroll-{scroll}')

        expand = self.eval(self.kwargs.get('expand'), context)
        if expand in self.POSSIBLE_EXPANDS:
            values['class'].append(f'nj-header--expand-{expand}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-header {class}" {props}>
  <div class="nj-header__group">
    {tmpl_head}
    <nav class="container">
      <div class="nj-header__nav-burger" aria-label="menu"
          aria-expanded="false">
        <button><div></div></button>
      </div>
      {tmpl_head_logosm}
      <ul class="nj-header__nav nj-header__nav--panel">
        {child}
      </ul>
      {slot_search}
    </nav>
  </div>
</{astag}>
"""
        return self.format(template, values, context)


    def render_tmpl_head(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not (values['logo_src'] or 'head_first' in self.slots or\
                'head_last' in self.slots):
            return ''
        tmpl = """
<div class="nj-header__head">
  {slot_head_first}
  {tmpl_head_logo}
  {slot_head_last}
</div>
<hr class="m-0">
"""
        return self.format(tmpl, values, context)


    def render_tmpl_head_logo(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not values['logo_src']:
            return ''
        tmpl = """
<a href="{href}" class="nj-header__logo">
  <img src="{logo_src}" alt="{logo_alt}" width="{logo_width}"
      height="{logo_height}">
</a>
"""
        return tmpl.format(**values)


    def render_tmpl_head_logosm(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not values['logosm_src']:
            return ''
        tmpl = """
<div class="nj-header__nav-logo--reduced">
  <a href="{href}">
    <img src="{logosm_src}" alt="{logo_alt}" width="{logosm_width}"
        height="{logosm_height}">
  </a>
</div>
"""
        return tmpl.format(**values)


class HeaderHeadLink(Node):
    """Header language link component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('active',)
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('nj-header__head-link--active')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-header__head-link {class}" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values)


class HeaderMenu(Node):
    """Header nav item component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('submenu',)
    "Named children."
    NODE_PROPS = ('active',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('wrapper',)
    "Prepare xxx_class and xxx_props values."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if self.eval(self.kwargs.get('active'), context):
            values['wrapper_class'].append('active')


    def after_prepare(self, values, context):
        """Simplifying values meant for rendering templates.
        """
        super().after_prepare(values, context)

        slot = self.slots.get('submenu')
        if slot:
            slot.kwargs.setdefault('label', values['child'])


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-header__nav-item {wrapper_class}" {wrapper_props}>
  <{astag} class="nj-header__nav-link {class}" {props}>
    {child} {tmpl_subarrow}
  </{astag}>
  {slot_submenu}
</li>
"""
        return self.format(template, values, context)


    def render_slot_submenu(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<div class="nj-header__menu nj-header__nav--panel">
  <a class="nj-header__menu-return">
    <i class="nj-header__menu-arrow-left material-icons md-24">
      keyboard_arrow_left
    </i>
    {label}
  </a>
  {child}
</div>
"""
        return tmpl.format(**values)


    def render_tmpl_subarrow(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not self.slots:
            return ''
        return """
<i class="nj-header__menu-arrow-right material-icons md-24">
  keyboard_arrow_right
</i>
"""


class HeaderMenuTag(HeaderMenu):
    """Header nav item as Tag component
    """
    NODE_PROPS = ('color', *HeaderMenu.NODE_PROPS)
    "Extended Template Tag arguments."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        super().prepare(values, context)

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-tag--{color}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-header__nav-item {wrapper_class}" {wrapper_props}>
  <{astag} class="nj-tag {class}" {props}>
    {child}
  </{astag}>
</li>
"""
        return self.format(template, values)


class HeaderSubmenu(Node):
    """Header submenu navigation component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<ul class="nj-header__sub-nav">
  <li>
    <a href="#" class="nj-header__menu-title" aria-label="open"
        aria-expanded="false">
      {label}
      <i class="nj-header__menu-arrow-right material-icons md-24">
        keyboard_arrow_right
      </i>
    </a>
    <ul class="nj-header__nav--panel">
      <li>
        <a class="nj-header__menu-return">
          <i class="nj-header__menu-arrow-left material-icons md-24">
            keyboard_arrow_left
          </i>
          {label}
        </a>
      </li>
      {child}
    </ul>
  </li>
</ul>
"""
        return self.format(template, values)


class HeaderMenuLink(Node):
    """Header nav item component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li>
  <{astag} class="nj-header__menu-link {class}" {props}>
    {child}
  </{astag}>
</li>
"""
        return self.format(template, values)


class HeaderSearch(Node):
    """Header search component
    """
    NODE_PROPS = ('action', 'method', 'id', 'color')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('form', 'icon')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_search'] = _("Search")
        values['txt_placeholder'] = _("Enter your query...")
        values['txt_close'] = _("Close")

        act = self.eval(self.kwargs.get('action'), context)
        if act:
            values['form_props'].append(('action', act))
        method = self.eval(self.kwargs.get('method'), context)
        if method:
            values['form_props'].append(('method', method))

        color = self.eval(self.kwargs.get('color', 'brand'), context)
        if color in self.POSSIBLE_COLORS:
            values['icon_class'].append(f'nj-icon-material--{color}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<a class="nj-header__search-icon" data-toggle="collapse"
    data-target="#collapse-search-bar-header" aria-expanded="false"
    aria-controls="collapse-search-bar-header">
  <i class="material-icons nj-icon-material {icon_class}" {icon_props}>
    search
  </i>
</a>
<form class="nj-header__search nj-collapse {form_class}" id="{id}" {form_props}>
  <input class="nj-form-control nj-navbar__search-input {class}" type="text"
      id="{id}-input" placeholder="{txt_placeholder}" {props}>
  <button type="submit" class="nj-btn nj-navbar__search-button">
    {txt_search}
  </button>
  <a data-target="#collapse-search-bar-header" class="nj-header__close"
      aria-label="{txt_close}" data-toggle="collapse">
    <i class="material-icons nj-icon-material {icon_class}" {icon_props}>
      close
    </i>
  </a>
</form>
"""
        return self.format(template, values)


components = {
    'Header': Header,
    'H_HeadLink': HeaderHeadLink,
    'H_Menu': HeaderMenu,
    'H_MenuTag': HeaderMenuTag,
    'H_Link': HeaderMenuLink,
    'H_Submenu': HeaderSubmenu,
    'H_Search': HeaderSearch,
}



"""
Tag
===

See: https://www.engie.design/fluid-design-system/components/tag/

Tags are used to show the criteria used to filter information. They can be
combined and used in every color of ENGIE’s palette.
Tags are used to visually label UI objects and elements for quick recognition.
For example, we can use them on cards, on tables, on form, etc. 
"""

from django.utils.translation import gettext as _
#-
from .base import COLORS, Node

class Tag(Node):
    """Tag component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    MODES = ('default', 'anchor')
    "Available variants."
    NODE_PROPS = ('delete', 'disabled', 'href', 'size', 'color', 'inversed')
    "Extended Template Tag arguments."
    POSSIBLE_SIZES = ('sm', 'lg')
    "Possible values for size argument."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    CLASS_AND_PROPS = ('child', 'button')
    "Prepare xxx_class and xxx_props values."

    # Parent Tags can set the arguments of their children Tags, in effect
    # changing their appearance.
    CATCH_PROPS = ('tag_kwargs',)

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_remove'] = _("Remove tag")

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-tag--{size}')

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-tag--{color}')

        href = self.eval(self.kwargs.get('href'), context)
        disabled =  self.eval(self.kwargs.get('disabled'), context)

        if self.mode == 'anchor':
            values['props'].append(('href', href))

            if disabled:
                values['class'].append('nj-tag--disabled')
                values['props'].append(('aria-disabled', 'true'))

        elif disabled:
            values['class'].append('nj-tag--disabled')
            values['child_tag'] = 'a'
            values['child_class'].append('nj-tag__text')
            values['child_props'].append(('role', 'link'))
            values['child_props'].append(('aria-disabled', 'true'))
        elif href:
            values['child_tag'] = 'a'
            values['child_props'].append(('href', href))
            values['child_class'].append('nj-tag__link')
        else:
            values['child_tag'] = 'span'
            values['child_class'].append('nj-tag__text')

        if not self.eval(self.kwargs.get('inversed'), context):
            values['button_class'].append('nj-icon-btn--secondary')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-tag {class}" {props}>
  <{child_tag} class="{child_class}" {child_props}>{child}</{child_tag}>
  {slot_icon}
  {tmpl_delete}
</{astag}>
"""
        return self.format(template, values, context)


    def render_anchor(self, values, context):
        """Html output of the component
        """
        template = """
<a class="nj-tag {class}" {props}>
  <span class="nj-tag__text">{child}</span>
  {slot_icon}
  {tmpl_delete}
</a>
"""
        return self.format(template, values, context)


    def render_tmpl_delete(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not self.eval(self.kwargs.get('delete'), context):
            return ''

        template = """
<button type="button" class="nj-tag__close nj-icon-btn nj-icon-btn--sm {button_class}">
  <span class="nj-sr-only">{txt_remove} {child}</span>
  <span aria-hidden="true" class="nj-icon-btn__icon material-icons">close</span>
</button>
"""
        return template.format(**values)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<span class="nj-tag__icon {class}" aria-hidden="true" {props}>
  {child}
</span>
"""
        return tmpl.format(**values)


components = {
    'Tag': Tag,
}



"""
Tabs
====

See: https://www.engie.design/fluid-design-system/components/tabs/

Tabs organise content across different screens with a simple navigation.
"""
from django.utils.translation import gettext as _
from .base import Node

class Tabs(Node):
    """Tabs component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('buttons',)
    "Named children."
    NODE_PROPS = ('style',)
    "Extended Template Tag arguments."
    POSSIBLE_STYLES = ('compact', 'spacious', 'stretched')
    "Possible values for style argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_buttons'] = _("Tab system label")

        style = self.eval(self.kwargs.get('style'), context)
        if style in self.POSSIBLE_STYLES:
            values['class'].append(f'nj-tab--{style}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-tab {class}" {props}>
  <div class="nj-tab__items" role="tablist" aria-label="{txt_buttons}">
    {slot_buttons}
  </div>
  <div style="padding-top: var(--nj-size-space-16);">
    {child}
  </div>
</{astag}>
"""
        return self.format(template, values, context)


class TabsButton(Node):
    """Tabs button component
    """
    NODE_PROPS = ('active', 'target_id')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        target = self.eval(self.kwargs.get('target_id'), context)
        if target:
            values['props'].append(('aria-controls', target))

        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('nj-tab__item--active')
            values['props'].append(('aria-selected', 'true'))
            values['props'].append(('tabindex', '0'))
        else:
            values['props'].append(('aria-selected', 'false'))
            values['props'].append(('tabindex', '-1'))

        if self.eval(self.kwargs.get('disabled'), context):
            values['props'].append(('disabled', True))


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-tab__item {class}" role="tab" {props}>
  {label}
</{astag}>
"""
        return self.format(template, values, context)


class TabsPanel(Node):
    """Tabs panel component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('active', 'trigger_id')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        trigger = self.eval(self.kwargs.get('trigger_id'), context)
        if trigger:
            values['props'].append(('aria-labelledby', trigger))

        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('nj-tab__content--active')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-tab__content {class}" role="tabpanel" tabindex="0" {props}>
  {child}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Tab': Tabs,
    'T_Btn': TabsButton,
    'T_Panel': TabsPanel,
}



"""
Form
====

See: https://www.engie.design/fluid-design-system/components/form/

Forms are used to send and collect data.
"""
from django.forms.widgets import (PasswordInput as PasswordWidget,
        Textarea as TextareaWidget)
from django.utils.translation import gettext as _
#-
from .base import FormNode

class TextInput(FormNode):
    """Form text item component
    """
    NODE_PROPS = ('floating', 'size', 'disabled')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'wrapper')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_SIZES = ('sm', 'lg', 'xl')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        if not self.eval(self.kwargs.get('floating'), context):
            values['wrapper_class'].append('nj-form-item--static')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['wrapper_class'].append(f'nj-form-item--{size}')

        if self.eval(self.kwargs.get('disabled'), context):
            values['wrapper_class'].append('nj-form-item--disabled')


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('nj-form-item__field')

        if self.eval(self.kwargs.get('floating'), context):
            props['placeholder'] = ''


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<div class="nj-form-item nj-form-item--error {wrapper_class}" {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
  </div>
  {tmpl_help}
  {tmpl_errors}
</div>
"""
        else:
            template = """
<div class="nj-form-item {wrapper_class}" {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


class PasswordInput(TextInput):
    """Form password item component
    """
    widget_class = PasswordWidget

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        super().prepare(values, context)

        values['txt_hide'] = _("Hide password")
        values['txt_show'] = _("Show password")
        values['txt_hidden'] = _("Password is hidden")
        values['txt_visible'] = _("Password is visible")


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<div class="nj-form-item nj-form-item--error nj-form-item--password {wrapper_class}"
    {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    <button type="button" aria-pressed="false"
        class="nj-form-item__password-button nj-icon-btn nj-icon-btn--lg nj-icon-btn--secondary">
      <span class="nj-sr-only" data-password-button-label-show="{txt_show}"
          data-password-button-label-hide="{txt_hide}"></span>
      <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
        visibility
      </span>
    </button>
    <p class="nj-sr-only nj-form-item__password-notice" aria-live="polite"
        aria-atomic="true" data-password-notice-is-visible="{txt_visible}"
        data-password-notice-is-hidden="{txt_hidden}"></p>
  </div>
  {tmpl_help}
  {tmpl_errors}
</div>
"""
        else:
            template = """
<div class="nj-form-item nj-form-item--password {wrapper_class}"
    {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    <button type="button" aria-pressed="false"
        class="nj-form-item__password-button nj-icon-btn nj-icon-btn--lg nj-icon-btn--secondary">
      <span class="nj-sr-only" data-password-button-label-show="{txt_show}"
          data-password-button-label-hide="{txt_hide}"></span>
      <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
        visibility
      </span>
    </button>
    <p class="nj-sr-only nj-form-item__password-notice" aria-live="polite"
        aria-atomic="true" data-password-notice-is-visible="{txt_visible}"
        data-password-notice-is-hidden="{txt_hidden}"></p>
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


class TextInputIcon(TextInput):
    """Form text item component with icon
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."

    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<div class="nj-form-item nj-form-item--error {wrapper_class}" {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    {slot_icon}
  </div>
  {tmpl_help}
  {tmpl_errors}
</div>
"""
        else:
            template = """
<div class="nj-form-item {wrapper_class}" {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    {slot_icon}
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<span class="nj-form-item__icon {class}" aria-hidden="true" {props}>
  {child}
</span>
"""
        return tmpl.format(**values)


class Textarea(TextInput):
    """Form textarea item component
    """
    widget_class = TextareaWidget

    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<div class="nj-form-item nj-form-item--textarea nj-form-item--error {wrapper_class}"
    {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    {slot_icon}
  </div>
  {tmpl_help}
  {tmpl_errors}
</div>
"""
        else:
            template = """
<div class="nj-form-item nj-form-item--textarea {wrapper_class}"
    {wrapper_props}>
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}
    {slot_icon}
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


class NumberInput(FormNode):
    """Form number item component
    """
    NODE_PROPS = ('static', 'size', 'format')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'wrapper', 'button')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_SIZES = ('sm', 'lg', 'xl')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_decr'] = _("- Decrement")
        values['txt_incr'] = _("+ Increment")

        if self.eval(self.kwargs.get('static'), context):
            values['wrapper_class'].append('nj-form-item--static')

        if self.bound_field.errors:
            values['group_labelled_by'] = '{id} {id}-error'.format(
                    id=values['id'])
        elif self.bound_field.help_text:
            values['group_labelled_by'] = '{id} {id}-hint'.format(
                    id=values['id'])
        else:
            values['group_labelled_by'] = values['id']

        if self.eval(self.kwargs.get('disabled'), context):
            values['wrapper_class'].append('nj-form-item--disabled')
            values['button_props'].append(('disabled', True))

        if self.eval(self.kwargs.get('readonly'), context):
            values['button_props'].append(('disabled', True))

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['wrapper_class'].append(f'nj-form-item--{size}')

        fmt = self.eval(self.kwargs.get('format'), context)
        if fmt:
            values['wrapper_props'].append(('data-live-zone-format', fmt))


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('nj-form-item__field')
        props['inputmode'] = 'numeric'


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.bound_field.errors:
            template = """
<div class="nj-form-item nj-form-item--error nj-form-item--input-number {wrapper_class}"
    {wrapper_props}>
  <div aria-labelledby="{group_labelled_by}" class="nj-form-item__field-wrapper"
      role="group">
    {tmpl_decr}
    {tmpl_element}
    {tmpl_label}
    {tmpl_incr}
    <div aria-atomic="true" aria-live="polite" class="nj-sr-only"></div>
  </div>
  {tmpl_help}
  {tmpl_errors}
</div>
"""
        else:
            template = """
<div class="nj-form-item nj-form-item--input-number {wrapper_class}"
    {wrapper_props}>
  <div aria-labelledby="{group_labelled_by}" class="nj-form-item__field-wrapper"
      role="group">
    {tmpl_decr}
    {tmpl_element}
    {tmpl_label}
    {tmpl_incr}
    <div aria-atomic="true" aria-live="polite" class="nj-sr-only"></div>
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_decr(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<button type="button"
    class="nj-icon-btn nj-icon-btn--secondary nj-form-item__decrement-button {button_class}"
    {button_props}>
  <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
    remove
  </span>
  <span class="nj-sr-only">{txt_decr}</span>
</button>
"""
        return self.format(template, values)


    def render_tmpl_incr(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<button type="button"
    class="nj-icon-btn nj-icon-btn--secondary nj-form-item__increment-button {button_class}"
    {button_props}>
  <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
    add
  </span>
  <span class="nj-sr-only">{txt_incr}</span>
</button>
"""
        return self.format(template, values)


components = {
    'NumberInput': NumberInput,
    'PasswordInput': PasswordInput,
    'TextInput': TextInput,
    'TextInputIcon': TextInputIcon,
    'Textarea': Textarea,
}



"""
Floating Action Button
======================

See: https://www.engie.design/fluid-design-system/components/fab/

Floating Action Buttons are just like buttons but they are not static. They
follow the journey of the user and display contextual actions at the perfect
moment. They are useful for mobile navigation. 
"""

from .base import Node

class Fab(Node):
    """Floating action button component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('menu',)
    "Named children."
    NODE_PROPS = ('disabled', 'size', 'placement')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('list', 'wrapper')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_SIZES = ('sm',)
    "Possible values for size argument."
    POSSIBLE_PLACEMENTS = ('right',)
    "Possible values for placement argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-fab--{size}')
            values['list_class'].append(f'nj-fab__actions--{size}')
        placement = self.eval(self.kwargs.get('placement'), context)
        if placement in self.POSSIBLE_PLACEMENTS:
            values['wrapper_props'].append(('data-placement', placement))

        if self.eval(self.kwargs.get('disabled'), context):
            values['class'].append('disabled')
            values['props'].append(('disabled', ''))


    def render_default(self, values, context):
        """Html output of the component
        """
        if 'menu' in self.slots:
            template = """
<div class="nj-fab-menu {wrapper_class}" {wrapper_props}>
  <button type="button" class="nj-fab {class}" {props}>
    {child}
  </button>
  <ul class="nj-fab__actions {list_class}" {list_props}>
    {slot_menu}
  </ul>
</div>
"""
        else:
            template = """
<button type="button" class="nj-fab {class}" {props}>
  {child}
</button>
"""
        return self.format(template, values, context)


class FabItem(Node):
    """Floating action menu component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li class="nj-fab__item">
  <button type="button" class="nj-fab nj-fab--light nj-fab--sm {class}" {props}>
    {child}
  </button>
</li>
"""
        return self.format(template, values, context)


components = {
    'Fab': Fab,
    'FabItem': FabItem,
}



"""
Footer
======

See: https://www.engie.design/fluid-design-system/components/footer/

Footer is mainly used for links and legal information.
"""
from .base import Node

class Footer(Node):
    """Footer component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('banner', 'menu', 'social')
    "Named children."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<footer class="nj-footer {class}" role="contentinfo" {props}>
  <div class="container">
    {slot_banner}
    {slot_menu}
    <ul class="nj-footer__links">
      {child}
    </ul>
    {slot_social}
  </div>
</footer>
"""
        return self.format(template, values, context)


    def render_slot_banner(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<div class="nj-footer__baseline {class}" {props}>
  {child}
</div>
<hr>
"""
        return tmpl.format(**values)


    def render_slot_menu(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<div class="nj-footer__menu {class}" {props}>
  {child}
</div>
<hr>
"""
        return tmpl.format(**values)


    def render_slot_social(self, values, context):
        """Render html of the slot.
        """
        tmpl = """
<ul class="nj-footer__social {class}" {props}>
  {child}
</ul>
"""
        return tmpl.format(**values)


class FooterLink(Node):
    """Footer link component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li>
  <{astag} class="nj-link nj-link--contextual {class}" {props}>
    {child}
  </{astag}>
</li>
"""
        return self.format(template, values, context)


class FooterSocial(Node):
    """Footer social link component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<li>
  <{astag} class="nj-footer__social-link {class}" {props}>
    {child}
  </{astag}>
</li>
"""
        return self.format(template, values, context)


class FooterMenuSection(Node):
    """Footer menu section component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-footer__menu-section">
  {tmpl_label}
  <ul class="nj-footer__links-list">
    {child}
  </ul>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template
        """
        if not values.get('label'):
            return ''
        template = '<h2 class="nj-footer__links-list-title">{label}</h2>'
        return self.format(template, values)


components = {
    'Footer': Footer,
    'FooterLink': FooterLink,
    'FooterSocial': FooterSocial,
    'FooterMenu': FooterMenuSection,
}



"""
Status-indicator
================

Status indicators are dynamic pieces of information that should be used to
convey the status of a person, an object or a process. They do not require any
user actions to be updated, and are usually part of a larger component.
"""
from django.utils.translation import gettext as _
#-
from .base import Node

class StatusIndicator(Node):
    """Status indicator component
    """
    NODE_PROPS = ('status', 'size', 'nolabel')
    "Extended Template Tag arguments."
    POSSIBLE_STATUSES = ('offline', 'online', 'away', 'do-not-disturb', 'busy',
            'unknown', 'error', 'success', 'warning', 'in-progress', 'info')
    "Possible values for status argument."
    POSSIBLE_SIZES = ('sm', 'lg')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        status = self.eval(self.kwargs.get('status'), context)
        if status:
            values['class'].append(f'nj-status-indicator--{status}')
        if status == 'offline':
            values['txt_label'] =  _("Offline")
        elif status == 'online':
            values['txt_label'] =  _("Online")
        elif status == 'away':
            values['txt_label'] =  _("Away")
        elif status == 'do-not-disturb':
            values['txt_label'] =  _("Do not disturb")
        elif status == 'busy':
            values['txt_label'] =  _("Busy")
        elif status == 'unknown':
            values['txt_label'] =  _("Unknown")
        elif status == 'error':
            values['txt_label'] =  _("Error")
        elif status == 'success':
            values['txt_label'] =  _("Success")
        elif status == 'warning':
            values['txt_label'] =  _("Warning")
        elif status == 'in-progress':
            values['txt_label'] =  _("In progress")
        elif status == 'info':
            values['txt_label'] =  _("Info")
        else:
            values['txt_label'] =  _("Online")

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-status-indicator--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        if self.eval(self.kwargs.get('nolabel'), context):
            template = """
<{astag} aria-hidden="true" class="nj-status-indicator {class}" {props}>
  <div class="nj-status-indicator__svg"></div>
</{astag}>
"""
        else:
            template = """
<{astag} class="nj-status-indicator {class}" {props}>
  <div class="nj-status-indicator__svg"></div>
  <p class="nj-status-indicator__text">{txt_label}</p>
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'StatusIndicator': StatusIndicator,
}



"""
Autocomplete
============

See: https://www.engie.design/fluid-design-system/components/autocomplete/

Autocomplete provides automated assistance to fill in form field values. It
allows the user to have suggestions while typing in the field.
"""
import json
#-
from django.utils.html import escape
from django.utils.translation import gettext as _
#-
from .base import FormNode

class Autocomplete(FormNode):
    """Autocomplete component
    """
    NODE_PROPS = ('data',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_instruction'] = _("Use the UP / DOWN arrows to navigate "
                "within the suggestion list. Press Enter to select an option. "
                "On touch devices, use swipe to navigate and double tap to "
                "select an option")

        data = self.eval(self.kwargs.get('data'), context)
        if data:
            values['props'].append(('data-list', escape(json.dumps(data))))


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['role'] = 'combobox'
        props['aria-autocomplete'] = 'list'
        props['aria-controls'].append(f'{props["id"]}-list')
        props['aria-expanded'] = 'false'
        props['autocomplete'] = 'off'
        props['aria-describedby'] = f'{props["id"]}-autocomplete-instructions'
        props['class'].append('nj-form-item__field')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<p id="{id}-autocomplete-instructions" hidden>{txt_instruction}</p>
<div class="nj-form-item nj-form-item--autocomplete">
  <div class="nj-form-item__field-wrapper">
    {tmpl_element}
    {tmpl_label}

    <ul role="listbox" id="{id}-list" tabindex="-1"
        aria-label="Countries suggestions" hidden
        class="nj-form-item__list nj-list-group nj-list-group--no-border nj-list-group--sm">
      <li role="option" aria-selected="false" tabindex="-1"
          class="nj-list-group__item nj-list-group__item--clickable"/>
    </ul>

    <span aria-hidden="true" class="nj-form-item__icon material-icons">
      keyboard_arrow_down
    </span>
  </div>
</div>
"""
        return self.format(template, values, context)


components = {
    'Autocomplete': Autocomplete,
}



"""
Pagination
==========

See: https://www.engie.design/fluid-design-system/components/pagination/

Pagination helps users to navigate through multiple pages.
"""
from django.utils.translation import gettext as _
#-
from .base import Node

class Pagination(Node):
    """Pagination component
    """
    NODE_PROPS = ('links', 'current', 'total')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_previous'] = _("Previous page")
        values['txt_next'] = _("Next page")

        if values['label']:
            values['props'].append(('aria-label', values['label']))

        values['current'] = self.eval(self.kwargs.get('current', 1), context)
        values['total'] = self.eval(self.kwargs.get('total', 0), context)

        links_count = max(self.eval(self.kwargs.get('links', 7), context), 1)
        values['show_first_last'] = links_count >= 6

        first_num = 1
        last_num = values['total']
        distance = links_count - 1
        if values['show_first_last']:
            first_num += 1
            last_num -= 1
            distance -= 2

        left_distance = right_distance = int(distance / 2)
        if distance % 2:
            left_distance += 1
        start_num = max(first_num, values['current'] - left_distance)
        right_distance += left_distance - (values['current'] - start_num)
        end_num = min(values['current'] + right_distance, last_num)
        values['numbers'] = list(range(start_num, end_num + 1))
        if values['numbers']:
            if start_num != first_num:
                values['numbers'][0] = None
            if end_num != last_num:
                values['numbers'][-1] = None


    def render_default(self, values, context):
        """Html output of the component
        """
        if values['show_first_last']:
            template = """
<nav role="navigation" class="{class}" {props}>
  <ul class="nj-pagination">
    {tmpl_previous}
    {tmpl_first}
    {tmpl_numbers}
    {tmpl_last}
    {tmpl_next}
  </ul>
</nav>
"""
        else:
            template = """
<nav role="navigation" class="{class}" {props}>
  <ul class="nj-pagination">
    {tmpl_previous}
    {tmpl_numbers}
    {tmpl_next}
  </ul>
</nav>
"""
        return self.format(template, values, context)


    def render_tmpl_previous(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<li class="nj-pagination__item">
  <button type="button" class="nj-icon-btn nj-icon-btn--lg">
    <span class="nj-sr-only">{txt_previous}</span>
    <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
      chevron_left
    </span>
  </button>
</li>
"""
        return self.format(template, values)


    def render_tmpl_next(self, values, context):
        """Dynamically render a part of the component's template
        """
        template = """
<li class="nj-pagination__item">
  <button type="button" class="nj-icon-btn nj-icon-btn--lg">
    <span class="nj-sr-only">{txt_next}</span>
    <span aria-hidden="true" class="nj-icon-btn__icon material-icons">
      chevron_right
    </span>
  </button>
</li>
"""
        return self.format(template, values)


    def render_tmpl_first(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['total'] < 1:
            return ''
        template = """
<li class="nj-pagination__item">
  <a class="nj-pagination__link" href="#" aria-label="Page 1">1</a>
</li>
"""
        return self.format(template, values, context)


    def render_tmpl_last(self, values, context):
        """Dynamically render a part of the component's template
        """
        if values['total'] < 2:
            return ''
        template = """
<li class="nj-pagination__item">
  <a class="nj-pagination__link" href="#" aria-label="Page {total}">{total}</a>
</li>
"""
        return self.format(template, values)


    def render_tmpl_numbers(self, values, context):
        """Dynamically render a part of the component's template
        """
        tpl_item = """
<li class="nj-pagination__item">
  <a class="nj-pagination__link" href="#" aria-label="Page {num}">{num}</a>
</li>
"""
        tpl_item_active = """
<li class="nj-pagination__item nj-pagination__item--active">
  <a class="nj-pagination__link" href="#" aria-current="true"
      aria-label="Page {num}">
    {num}
  </a>
</li>
"""
        tpl_item_more = """
<li class="nj-pagination__item">
  <span class="nj-pagination__more material-icons" aria-hidden="true">
    more_horiz
  </span>
  <span class="nj-sr-only">...</span>
</li>
"""
        items = []
        for num in values['numbers']:
            if num is None:
                items.append(tpl_item_more)
            elif num == values['current']:
                items.append(tpl_item_active.format(num=num))
            else:
                items.append(tpl_item.format(num=num))
        return '\n'.join(items)


components = {
    'Pagination': Pagination,
}



"""
Icon
====

Icons are symbols that represent objects and concepts visually. They help users
understand the message without text and should be as informative as possible.
They shouldn't be used to "decorate" the interface. They communicate messages in
the simplest way.
"""
from .base import COLORS, Node

class Icon(Node):
    """Icon component
    """
    NODE_PROPS = ('color', 'size')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'span'
    "Rendered HTML tag."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    POSSIBLE_SIZES = ('sm', 'lg', 'xl', 'xxl')
    "Possible values for size argument."

    # Parent Tags can set the arguments of their children Tags, in effect
    # changing their appearance.
    CATCH_PROPS = ('icon_kwargs',)

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-icon-material--{size}')

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-icon-material--{color}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} aria-hidden="true" class="material-icons nj-icon-material {class}"
    {props}>
  {label}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Icon': Icon,
}



"""Django application configuration for fluid_design.
"""
from django.apps import AppConfig

class FluidDesignConfig(AppConfig):
    """Django application configuration for fluid_design.
    """
    name = 'fluid_design'



"""Custom form widgets.
"""
from django.forms.widgets import (CheckboxInput, Input, NumberInput, Textarea,
        TextInput)

class CustomNameInput(Input):
    """Input widget where you can change html attribute: name.
    """
    def get_context(self, name, value, attrs):
        """Modify so we can override name attribute.
        """
        name =  attrs.pop('name', name)
        value =  attrs.pop('value', value)
        return super().get_context(name, value, attrs)


class CustomCheckboxInput(CustomNameInput, CheckboxInput):
    """Checkbox widget where you can change html attribute: name.
    """
    def use_required_attribute(self, initial):
        """Checkbox that is part of a fieldset doesn't need required attribute
        """
        return False


class CustomTextarea(CustomNameInput, Textarea):
    """Textarea widget where you can change html attribute: name.
    """


class CustomTextInput(CustomNameInput, TextInput):
    """TextInput widget where you can change html attribute: name.
    """


class CustomRangeInput(CustomNameInput, NumberInput):
    """TextInput widget where you can change html attribute: name.
    """
    input_type = 'range'



"""
Segmented control
=================

See: https://www.engie.design/fluid-design-system/components/segmented-control/

Segmented controls are helpful to show closely-related options users can choose
from. They can be used to switch views for example.
"""
from .base import clean_attr_value, Node

class SegmentedControl(Node):
    """Segmented control component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'compact')
    "Available variants."
    NODE_PROPS = ('value', 'size')
    "Extended Template Tag arguments."
    POSSIBLE_SIZES = ('sm', 'lg')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        context['segmented_control_button_kwargs'] = {}

        if self.mode == 'compact':
            context['segmented_control_button_kwargs']['compact'] = True

        if values['label']:
            values['props'].append(('aria-label', values['label']))

        value = self.eval(self.kwargs.get('value'), context)
        if value:
            values['props'].append(('data-value', value))

            context['segmented_control_button_kwargs']['selected'] = value

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-segmented-control--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<div class="nj-segmented-control {class}" role="group" {props}>
  {child}
</div>
"""
        return self.format(template, values, context)


    def render_compact(self, values, context):
        """Html output of the component
        """
        return self.render_default(values, context)


class SegmentedControlButton(Node):
    """Segmented control button component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('value', 'selected', 'compact')
    "Extended Template Tag arguments."

    # Parent Tags can set the arguments of their children Tags, in effect
    # changing their appearance.
    CATCH_PROPS = ('segmented_control_button_kwargs',)

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        context['icon_icon_kwargs'] = {
            'class': 'nj-segmented-control-btn__icon',
        }

        values['value'] = self.eval(self.kwargs.get('value'), context)
        values['compact'] = self.eval(self.kwargs.get('compact'), context)


    def after_prepare(self, values, context):
        """Simplifying values meant for rendering templates.
        """
        cleaned_child = clean_attr_value(values['child'])
        if not values['value']:
            values['value'] = cleaned_child
        if values['value'] == self.eval(self.kwargs.get('selected'),
                context):
            values['props'].append(('aria-pressed', 'true'))
        else:
            values['props'].append(('aria-pressed', 'false'))

        if values['compact']:
            values['props'].append(('title', cleaned_child))

        super().after_prepare(values, context)


    def render_default(self, values, context):
        """Html output of the component
        """
        if 'icon' in self.slots:
            if values['compact']:
                template = """
<button class="nj-segmented-control-btn {class}" type="button"
    data-value="{value}" {props}>
  {slot_icon}
  <span class="nj-sr-only">{child}</span>
</button>
"""
            else:
                template = """
<button class="nj-segmented-control-btn {class}" type="button"
    data-value="{value}" {props}>
  {slot_icon}
  <span>{child}</span>
</button>
"""
        else:
            template = """
<button class="nj-segmented-control-btn {class}" type="button"
    data-value="{value}" {props}>
  {child}
</button>
"""
        return self.format(template, values, context)


components = {
    'SegmentedControl': SegmentedControl,
    'SegmentedControlBtn': SegmentedControlButton,
}



"""
Collapse
========

See: https://www.engie.design/fluid-design-system/components/collapse/

Collapses allow users to toggle the visibility of a content

How it works
------------

The collapse JavaScript plugin is used to show and hide content. Buttons or
anchors are used as triggers that are mapped to specific elements you toggle.
Collapsing an element will animate the height from its current value to 0.
Given how CSS handles animations, you cannot use padding on a .nj-collapse
element. Instead, use the class as an independent wrapping element.
"""

from .base import Node

class Collapse(Node):
    """Collapse component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<{astag} class="nj-collapse {class}" {props}>
  <div class="nj-card nj-card__body">
    {child}
  </div>
</{astag}>
"""
        return self.format(template, values, context)


class CollapseButton(Node):
    """Collapse button component
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'anchor')
    "Available variants."
    NODE_PROPS = ('target', 'controls')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        target = self.eval(self.kwargs.get('target'), context)
        controls = self.eval(self.kwargs.get('controls'), context)

        values['target'] = target
        if controls:
            values['controls'] = controls
        elif target and target.startswith('#'):
            values['controls'] = target[1:]
        else:
            values['controls'] = ''


    def render_default(self, values, context):
        """Html output of the component
        """
        template = """
<button class="nj-btn {class}" type="button" data-toggle="collapse"
    data-target="{target}" aria-expanded="false" aria-controls="{controls}"
    {props}>
  {child}
</button>
"""
        return self.format(template, values, context)


    def render_anchor(self, values, context):
        """Html output of the component
        """
        template = """
<a class="nj-btn" role="button" data-toggle="collapse"
    href="{target}" aria-expanded="false" aria-controls="{controls}" {props}>
  {child}
</a>
"""
        return self.format(template, values, context)


components = {
    'CollapseBtn': CollapseButton,
    'Collapse': Collapse,
}



"""
Spinner
=======

See: https://www.engie.design/fluid-design-system/components/spinner/

Spinner allows the user to know when the system is in progress and when he will
end. The spinner is used to indicate the current status of a loading screen or
a loading data.
"""
from django.utils.translation import gettext as _
#-
from .base import COLORS, Node

class Spinner(Node):
    """Spinner component
    """
    NODE_PROPS = ('color', 'size')
    "Extended Template Tag arguments."
    POSSIBLE_COLORS = COLORS
    "Possible values for color argument."
    POSSIBLE_SIZES = ('sm', 'md', 'xs', 'xxs')
    "Possible values for size argument."

    def prepare(self, values, context):
        """Prepare values for rendering the templates
        """
        values['txt_loading'] = _("Loading...")

        color = self.eval(self.kwargs.get('color'), context)
        if color in self.POSSIBLE_COLORS:
            values['class'].append(f'nj-spinner--{color}')

        size = self.eval(self.kwargs.get('size'), context)
        if size in self.POSSIBLE_SIZES:
            values['class'].append(f'nj-spinner--{size}')


    def render_default(self, values, context):
        """Html output of the component
        """
        if values['astag'] == 'span':
            template = """
<span aria-live="polite" aria-atomic="true" class="nj-spinner {class}" {props}>
  <span class="nj-sr-only">{txt_loading}</span>
  {label}
</span>
"""
        else:
            template = """
<{astag} aria-live="polite" aria-atomic="true" class="nj-spinner {class}" {props}>
  <p class="nj-sr-only">{txt_loading}</p>
  {label}
</{astag}>
"""
        return self.format(template, values, context)


components = {
    'Spinner': Spinner,
}
